'use strict';

exports.__esModule = true;
exports.ValidationLayer = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('./utils');

var formUtils = _interopRequireWildcard(_utils);

var _formConstants = require('./enums/formConstants');

var _feedbackStrategies = require('./enums/feedbackStrategies');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// TODO:  - Test stratigies
//        - Add button props
//        - Add async validations
//        - Add collections handling
//        - Add refs
//        - Add focus on first invalid node

var ValidationLayer = exports.ValidationLayer = function (_React$Component) {
  _inherits(ValidationLayer, _React$Component);

  function ValidationLayer(props, context) {
    _classCallCheck(this, ValidationLayer);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props, context));

    _this.resetComponent();

    formUtils.bindToContext(_this, ['resetComponent', 'setBluredField', 'handleDomChange', 'handleCustomChange', 'handleChange', 'handleDomBlur', 'handleCustomBlur', 'handleBlur', 'handleFieldValidation', 'validateField', 'handleSubmit', 'handleFailedFormValidation', 'performSubmit', 'handleSuccessPostSubmitAction', 'handleFailurePostSubmitAction']);
    return _this;
  }

  ValidationLayer.prototype.componentWillMount = function componentWillMount() {
    var nextFormFieldsData = this.getFormFieldsData(this.props);
    this.setState(nextFormFieldsData);
  };

  ValidationLayer.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var nextFormFieldsData = this.getFormFieldsData(nextProps);
    this.setState(nextFormFieldsData);
  };

  ValidationLayer.prototype.resetComponent = function resetComponent() {
    this.isMultiEntities = this.props.isMultiEntities;

    this.changedFields = {};
    this.bluredFields = {};

    this.isSubmitting = false;
    this.formWasSubmitted = false;
  };

  ValidationLayer.prototype.setChangedField = function setChangedField(fieldId) {
    this.changedFields[fieldId] = true;
  };

  ValidationLayer.prototype.setBluredField = function setBluredField(fieldId) {
    this.bluredFields[fieldId] = true;
  };

  ValidationLayer.prototype.getFormFieldsData = function getFormFieldsData(props) {
    var formFields = {};

    for (var _iterator = props.fields, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var field = _ref;

      var _formUtils$getFieldId = formUtils.getFieldIdParts(props, field);

      var dataKey = _formUtils$getFieldId.dataKey;
      var entity = _formUtils$getFieldId.entity;
      var entityId = _formUtils$getFieldId.entityId;
      var attr = _formUtils$getFieldId.attr;


      if (this.isMultiEntities && !entityId) {
        throw new Error('`isMultiEntities` prop is set to `true`, but entity has no `id`');
      }

      var fieldId = formUtils.buildFieldId(dataKey, entityId, attr);
      var fieldStateId = formUtils.buildFieldDataStateId(fieldId);

      formFields[fieldStateId] = {};

      formFields[fieldStateId][_formConstants.formConstants.FIELD_ID_DATA_ATTRIBUTE] = fieldId;

      var omitDomId = formUtils.isDefined(field.omitDomId) ? field.omitDomId : props.omitDomId;

      if (!omitDomId) {
        var fieldDomId = formUtils.buildFieldDomId(dataKey, entityId, attr);
        formFields[fieldStateId].id = fieldDomId;
      }

      var fieldValue = formUtils.fetchProp(entity, attr);
      var domFieldValue = formUtils.isDefined(fieldValue) && !formUtils.isNull(fieldValue) ? fieldValue : '';

      var viewTransform = field.viewTransform || props.viewTransform;

      formFields[fieldStateId].value = viewTransform ? viewTransform(domFieldValue) : domFieldValue;

      formFields[fieldStateId].disabled = this.isSubmitting || field.disabled;

      var formHandlers = {};

      var _props$handlers = props.handlers;
      var onSubmit = _props$handlers.onSubmit;

      var sharedHandlers = _objectWithoutProperties(_props$handlers, ['onSubmit']); // eslint-disable-line

      var omitOnChange = formUtils.isDefined(field.omitOnChange) ? field.omitOnChange : props.omitOnChange;

      if (!omitOnChange) {
        formHandlers.onChange = this.handleDomChange;
      }

      var omitOnBlur = formUtils.isDefined(field.omitOnBlur) ? field.omitOnBlur : props.omitOnBlur;

      if (!omitOnBlur) {
        formHandlers.onBlur = this.handleDomBlur;
      }

      Object.assign(formFields[fieldStateId], sharedHandlers, field.handlers, formHandlers);
    }

    return formFields;
  };

  ValidationLayer.prototype.handleDomChange = function handleDomChange(e) {
    var domData = formUtils.getDataFromDom(e);
    this.handleChange(domData, e);
  };

  ValidationLayer.prototype.handleCustomChange = function handleCustomChange(fieldDomId, value) {
    var parsedFieldDomId = formUtils.parseFieldId(fieldDomId);
    var domData = Object.assign({}, parsedFieldDomId, { value: value });
    this.handleChange(domData);
  };

  ValidationLayer.prototype.handleChange = function handleChange(domData, e) {
    var props = this.props;


    if (!this.changedFields[domData.fieldId]) {
      this.setChangedField(domData.fieldId);
    }

    var field = formUtils.getField(props, domData);

    var filter = field.filter || props.filter;

    if (filter && domData.value && !filter(domData.value)) {
      return false;
    }

    var storeTransform = field.storeTransform || props.storeTransform;

    if (storeTransform) {
      domData.value = storeTransform(domData.value);
    }

    if (e) {
      var handleChange = formUtils.getHandler(field, props, 'onChange');
      handleChange(domData, {
        originalEvent: e,
        handleSubmit: this.handleSubmit
      });
    }

    return this.handleFieldValidation(field, domData, e);
  };

  ValidationLayer.prototype.handleDomBlur = function handleDomBlur(e) {
    var domData = formUtils.getDataFromDom(e);
    this.handleBlur(domData, e);
  };

  ValidationLayer.prototype.handleCustomBlur = function handleCustomBlur(fieldDomId, value, e) {
    var parsedFieldDomId = formUtils.parseFieldId(fieldDomId);
    var domData = Object.assign({}, parsedFieldDomId, { value: value });
    this.handleBlur(domData, e, true);
  };

  ValidationLayer.prototype.handleBlur = function handleBlur(domData, e) {
    var isCustom = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];
    var props = this.props;


    var field = formUtils.getField(props, domData);
    var handleBlur = formUtils.getHandler(field, props, 'onBlur');

    var storeTransform = field.storeTransform || props.storeTransform;

    if (storeTransform) {
      domData.value = storeTransform(domData.value);
    }

    if (!isCustom && handleBlur) {
      handleBlur(domData, e);
    }

    var feedbackStrategy = formUtils.getStrategy(this, field);

    if (feedbackStrategy === _feedbackStrategies.feedbackStrategies.ON_FIRST_BLUR || feedbackStrategy === _feedbackStrategies.feedbackStrategies.ON_BLUR_ONLY || feedbackStrategy === _feedbackStrategies.feedbackStrategies.ON_SUCCESS_OR_FIRST_BLUR) {
      this.handleFieldValidation(field, domData, e);
    }
  };

  ValidationLayer.prototype.handleFieldValidation = function handleFieldValidation(field, domData, e) {
    var nextState = this.validateField(field, domData, e);
    this.setState(nextState);
  };

  ValidationLayer.prototype.validateField = function validateField(field, domData, e) {
    return formUtils.applyStrategy(this, field, domData, e);
  };

  ValidationLayer.prototype.handleSubmit = function handleSubmit(e) {
    this.isSubmitting = true;

    if (!this.formWasSubmitted) {
      this.formWasSubmitted = true;
    }

    if (e && e.preventDefault) {
      e.preventDefault();
    }

    var nextFormFieldsData = this.getFormFieldsData(this.props);

    this.setState(nextFormFieldsData, this.handleFormValidation);
  };

  ValidationLayer.prototype.handleFormValidation = function handleFormValidation() {
    var props = this.props;


    var validationState = {};

    var isInvalid = false;

    for (var _iterator2 = props.fields, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
      var _ref2;

      if (_isArray2) {
        if (_i2 >= _iterator2.length) break;
        _ref2 = _iterator2[_i2++];
      } else {
        _i2 = _iterator2.next();
        if (_i2.done) break;
        _ref2 = _i2.value;
      }

      var field = _ref2;

      var _formUtils$getFieldId2 = formUtils.getFieldIdParts(props, field);

      var dataKey = _formUtils$getFieldId2.dataKey;
      var entity = _formUtils$getFieldId2.entity;
      var entityId = _formUtils$getFieldId2.entityId;
      var attr = _formUtils$getFieldId2.attr;


      var statuses = formUtils.getDefaultStatuses(props);

      var fieldId = formUtils.buildFieldId(dataKey, entityId, attr);
      var fieldStateId = formUtils.buildFieldValidationStateId(fieldId);

      var fieldValue = formUtils.fetchProp(entity, attr);

      var fieldValidationState = formUtils.normalizeValidationResults(field.validate, fieldValue, this.props);

      if (!fieldValidationState.valid) {
        fieldValidationState.status = fieldValidationState.status || statuses.error;
        isInvalid = true;
      } else if (fieldValue) {
        fieldValidationState.status = fieldValidationState.status || statuses.success;
      }

      validationState[fieldStateId] = fieldValidationState;
    }

    if (isInvalid) {
      this.handleFailedFormValidation(validationState);
    } else {
      this.performSubmit();
    }
  };

  ValidationLayer.prototype.handleFailedFormValidation = function handleFailedFormValidation(validationState) {
    this.setState(validationState, this.handleFailurePostSubmitAction);
  };

  ValidationLayer.prototype.performSubmit = function performSubmit() {
    this.props.handlers.onSubmit(this.handleSuccessPostSubmitAction);
  };

  ValidationLayer.prototype.handleSuccessPostSubmitAction = function handleSuccessPostSubmitAction() {
    this.resetComponent();

    var nextFormFieldsData = this.getFormFieldsData(this.props);
    var resetedState = formUtils.resetState(this.state, nextFormFieldsData);

    this.setState(resetedState);
  };

  ValidationLayer.prototype.handleFailurePostSubmitAction = function handleFailurePostSubmitAction() {
    this.isSubmitting = false;

    var nextFormFieldsData = this.getFormFieldsData(this.props);

    this.setState(nextFormFieldsData);
  };

  ValidationLayer.prototype.getFormProps = function getFormProps() {
    return { form: formUtils.createFormProps(this) };
  };

  ValidationLayer.prototype.render = function render() {
    var children = this.props.children;

    var formProps = this.getFormProps();

    return _react2.default.cloneElement(children, formProps);
  };

  return ValidationLayer;
}(_react2.default.Component);

ValidationLayer.propTypes = {
  children: _react.PropTypes.element.isRequired,

  dataKey: _react.PropTypes.string,
  dataType: _react.PropTypes.oneOf([_formConstants.formConstants.COLLECTION_DATA_TYPE]),

  handlers: _react.PropTypes.shape({
    onSubmit: _react.PropTypes.func.isRequired
  }).isRequired,

  feedbackStrategy: _react.PropTypes.oneOf([_feedbackStrategies.feedbackStrategies.INSTANT, _feedbackStrategies.feedbackStrategies.ON_SUCCESS, _feedbackStrategies.feedbackStrategies.ON_FIRST_BLUR, _feedbackStrategies.feedbackStrategies.ON_BLUR_ONLY, _feedbackStrategies.feedbackStrategies.ON_SUCCESS_OR_FIRST_BLUR, _feedbackStrategies.feedbackStrategies.ON_SUBMIT]),

  fields: _react.PropTypes.arrayOf(_react.PropTypes.shape({
    dataKey: _react.PropTypes.string, // Can be global
    dataType: _react.PropTypes.oneOf([// Can be global
    _formConstants.formConstants.COLLECTION_DATA_TYPE]),
    attr: _react.PropTypes.string.isRequired,
    validate: _react.PropTypes.func,
    validateAsync: _react.PropTypes.func,
    handlers: _react.PropTypes.object, // Can be global
    filter: _react.PropTypes.func, // Can be global
    viewTransform: _react.PropTypes.func, // Can be global
    storeTransform: _react.PropTypes.func, // Can be global
    omitDomId: _react.PropTypes.bool, // Can be global
    omitRef: _react.PropTypes.bool, // Can be global
    omitOnChange: _react.PropTypes.bool, // Can be global
    omitOnBlur: _react.PropTypes.bool, // Can be global

    feedbackStrategy: _react.PropTypes.oneOf([// Can be global
    _feedbackStrategies.feedbackStrategies.INSTANT, _feedbackStrategies.feedbackStrategies.ON_SUCCESS, _feedbackStrategies.feedbackStrategies.ON_FIRST_BLUR, _feedbackStrategies.feedbackStrategies.ON_BLUR_ONLY, _feedbackStrategies.feedbackStrategies.ON_SUCCESS_OR_FIRST_BLUR, _feedbackStrategies.feedbackStrategies.ON_SUBMIT])
  }).isRequired).isRequired,

  isMultiEntities: _react.PropTypes.bool,

  successStatus: _react.PropTypes.string,
  errorStatus: _react.PropTypes.string
};