'use strict';

exports.__esModule = true;
exports.createFormProps = createFormProps;

var _utils = require('../utils');

var formUtils = _interopRequireWildcard(_utils);

var _formConstants = require('../enums/formConstants');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function createFormProps(context) {
  var fields = {};

  for (var _iterator = Object.keys(context.state), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var fieldStateId = _ref;

    var _formUtils$parseField = formUtils.parseFieldStateId(fieldStateId);

    var dataType = _formUtils$parseField.dataType;
    var fieldId = _formUtils$parseField.fieldId;


    if (!fields[fieldId]) {
      fields[fieldId] = {};
    }

    if (dataType === _formConstants.formConstants.FIELD_DATA_STATE_ID_PREFIX) {
      fields[fieldId].props = context.state[fieldStateId];
    } else {
      fields[fieldId].message = context.state[fieldStateId].message;
      fields[fieldId].status = context.state[fieldStateId].status;
    }
  }

  var singleDataSource = formUtils.singleDataSource(context.props);

  function getPropsFor() {
    for (var _len = arguments.length, keyPath = Array(_len), _key = 0; _key < _len; _key++) {
      keyPath[_key] = arguments[_key];
    }

    var fieldId = formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
    return this.fields[fieldId].props;
  }

  function getPropsForCheckbox() {
    for (var _len2 = arguments.length, keyPath = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      keyPath[_key2] = arguments[_key2];
    }

    var fieldId = formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
    var stateProps = this.fields[fieldId].props;

    return Object.assign({}, stateProps, { checked: !!stateProps.value });
  }

  function getPropsForRadioButton(keyPath, value) {
    var fieldId = formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
    var stateProps = this.fields[fieldId].props;
    var radioButtonProps = Object.assign({}, stateProps);

    radioButtonProps.value = value;
    radioButtonProps.checked = stateProps.value === value;

    if (stateProps.id) {
      radioButtonProps.id = formUtils.getFieldDomIdFromKeyPath(singleDataSource, keyPath, value);
    }

    return radioButtonProps;
  }

  function getCustomPropsFor(keyPath, _ref2) {
    var value = _ref2.value;
    var comparator = _ref2.comparator;
    var disabled = _ref2.disabled;

    var fieldId = formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
    var stateProps = this.fields[fieldId].props;
    var customProps = Object.assign({}, stateProps);

    if (comparator) {
      customProps.checked = comparator(stateProps.value);
    }

    if (disabled) {
      customProps.disabled = disabled;
    }

    if (stateProps.id && value) {
      customProps.id = formUtils.getFieldDomIdFromKeyPath(singleDataSource, keyPath, value);
    }

    return customProps;
  }

  function getValidFor() {
    for (var _len3 = arguments.length, keyPath = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      keyPath[_key3] = arguments[_key3];
    }

    var fieldId = formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
    return this.fields[fieldId].valid;
  }

  function getMessageFor() {
    for (var _len4 = arguments.length, keyPath = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      keyPath[_key4] = arguments[_key4];
    }

    var fieldId = formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
    return this.fields[fieldId].message;
  }

  function getStatusFor() {
    for (var _len5 = arguments.length, keyPath = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
      keyPath[_key5] = arguments[_key5];
    }

    var fieldId = formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
    return this.fields[fieldId].status;
  }

  function getFieldIdFor() {
    for (var _len6 = arguments.length, keyPath = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
      keyPath[_key6] = arguments[_key6];
    }

    return formUtils.getFieldIdFromKeyPath(singleDataSource, keyPath);
  }

  function getDomIdFor() {
    for (var _len7 = arguments.length, keyPath = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
      keyPath[_key7] = arguments[_key7];
    }

    return formUtils.getFieldDomIdFromKeyPath(singleDataSource, keyPath);
  }

  function getDomIdWithValue(keyPath, value) {
    if (!Array.isArray(keyPath)) {
      throw new Error('`keyPath` passed to `fromFields.getFieldDomId` must be an Array');
    }

    return formUtils.getFieldDomIdFromKeyPath(singleDataSource, keyPath, value);
  }

  return {
    fields: fields,

    getPropsFor: getPropsFor,
    getPropsForCheckbox: getPropsForCheckbox,
    getPropsForRadioButton: getPropsForRadioButton,
    getCustomPropsFor: getCustomPropsFor,
    getValidFor: getValidFor,
    getMessageFor: getMessageFor,
    getStatusFor: getStatusFor,
    getFieldIdFor: getFieldIdFor,
    getDomIdFor: getDomIdFor,
    getDomIdForRadioButton: getDomIdWithValue,
    getCustomDomIdFor: getDomIdWithValue,

    handleChange: context.handleCustomChange,
    handleBlur: context.handleCustomBlur,
    handleSubmit: context.handleSubmit
  };
}